<?php

namespace Drupal\ephoto_dam_field\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\TraversableTypedDataInterface;

/**
 * Plugin implementation of the Ephoto Dam field type
 *
 * @FieldType(
 *   id = "ephoto_dam_field",
 *   label = @Translation("Ephoto Dam Field"),
 *   description = @Translation("Stores a Ephoto Dam URL and then outputs some embed code"),
 *   category = @Translation("Media"),
 *   default_widget = "ephoto_dam_field_widget",
 *   default_formatter = "ephoto_dam_field_formatter",
 *   constraints = {"EphotoDamFieldValidation" = {}} 
 * )
 */
class EphotoDamField extends FieldItemBase {

    /**
     * {@inheritdoc}
     */
    public function __construct($definition, $name = NULL, TraversableTypedDataInterface $parent = NULL) {
        parent::__construct($definition, $name, $parent);
    }

    /**
     * {@inheritdoc}
     */
    public static function createInstance($definition, $name = NULL, TraversableTypedDataInterface $parent = NULL) {
        return new static($definition, $name, $parent);
    }

    /**
     * {@inheritdoc}
     */
    public static function schema(FieldStorageDefinitionInterface $field_definition) {
        return [
            'columns' => [
                'url' => [
                    'type' => 'varchar',
                    'length' => 256
                ],
                'image_size' => [
                    'type' => 'varchar',
                    'length' => 9
                ],
                'version' => [
                    'type' => 'varchar',
                    'length' => 20 
                ]
            ]
        ];
    } 

    /**
     * {@inheritdoc}
     */
    public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
        $properties['url'] = DataDefinition::create('string')
            ->setLabel(t('Url'));

        $properties['image_size'] = DataDefinition::create('string')
            ->setLabel(t('Image size'));

        $properties['version'] = DataDefinition::create('string')
            ->setLabel(t('Version'));
        
        return $properties;
    }

    /**
     * {@inheritdoc}
     */
    public function isEmpty() {
        $url = $this->get('url')->getValue();
        $image_size = $this->get('image_size')->getValue();
        $version = $this->get('version')->getValue();

        return empty($url) && empty($image_size) && empty($version);
    }

    /**
     * {@inheritdoc}
     */
    public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
        $form = [];

        $form['server_url'] = [
            '#title' => t('Server URL'),
            '#type' => 'url',
            '#default_value' => $this->getSetting('server_url'),
            '#description' => t('Please enter the URL of your Ephoto Dam software'),
            '#required' => true,
            '#attributes' => [
                'class' => ['form-text', 'ephoto-dam-field-settings-url'],
                'placeholder' => t('https://ephoto.mycompany.com/')
            ],
            '#size' => 50,
        ];
        
        $form['version_support'] = [
                '#title' => t('Version support'),
                '#description' => t('An url will be automatically associated with each version with the same label in Ephoto Dam') . '.',
                '#type' => 'checkbox',
                '#default_value' => $this->getSetting('version_support'),
                '#attributes' => [
                    'class' => ['ephoto-dam-field-settings-version-support'],
                    'onclick' => 'Drupal.ephotoDamField.enableVersionSupport(this)'
                ]
        ];

        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public static function defaultFieldSettings() {
        return [
            'server_url' => '',
            'version_support' => false
        ];
    }  
}
